#! /usr/bin/perl -w
#
#  Input file format
#     The procesetup input files consist of lines comprised of one or more
#     fields separated by blanks or tabs. Fields containing white-space 
#     characters should be enclosed in braces ("{}"). The configuration line 
#     may be continued over multiple text lines by ending a non-terminal 
#     line with a backslash (\). Comments start with a hash-mark (#) and 
#     continue to the end of the line. 
#
#  Process list file.
#     The ProcList file contains one configuration line for each process to
#     be run. The configuration line contains the following fields:
#
#       1) Process name
#       2) CPU usage
#       3) Command to be executed by the process
#       4) Node assignment constraints
#       5) Procmgr tag
#       6) Files used by the process
#
#    The "node assignment constraints" field is a perl expression that 
#    evaluates to true if the process may be run on a given node. The
#    symbols in this expression are substituted with values defined in
#    the node list on a node-by-node basis.
#
#  ProcList internal storage
#    0) Process name
#    1) Process cp requirement
#    2) process command as "cmd args..."
#    3) process constraints "{expr1 && ... && exprn}"
#    4) procmgt tags as "procmgr{-cd:xxx -when:xxx}"
#    5) process file list as "files{file-1,file-2,...,file-n}"
#    6) assigned node
#
#--- Define process list database subroutines
#
$prefix="/usr";
$exec_prefix="${prefix}";
require "${exec_prefix}/bin/procmgt_files.pl";
#
# useful constant as a function
sub proc_ncol() {
    return 7;
}

#===========================================================================
#
#    Subroutine:  GetProcs
#
#    Arguments:   None
#
#    Global variables:
#                 input: $FileBase, $Debug
#                 inout: @ProcList
#                 
#
#    Returns:     void
#
#    Purpose:     Read list of nodes.
#
#===========================================================================
sub GetProcs {
    my $File = getFileName("ProcList");
    return if (! $File);
    open(ProcFile, $File);
    print (tStamp(), " Reading $File\n");
    while (defined($Line=proc_read(\*ProcFile))) {
	chomp $Line;
	$Line =~ s/(\s)+/ /g;
	my @Entry = ();
	while (! $Line =~ /^\s*$/) {
	    $Line =~ s/^ //;
	    if ($Line =~ s/^\{([^\}]*)\}// ) {
		push(@Entry, "$1");
	    } else {
		$Line =~ s/^([^ ]+)//;
		push(@Entry, "$1");
	    }
	}
	while (@Entry < proc_ncol()) {
	    push(@Entry, "");
	}
	if ($Debug) {
	    print("-------------------------------------\n") if (@ProcList==0);
	    print("Process Name: $Entry[0]\n",
		  "CPU reqired:  $Entry[1]\n",
		  "Command:      $Entry[2]\n",
		  "Constraints:  $Entry[3]\n",
		  "Tags:         $Entry[4]\n",
		  "Files:        $Entry[5]\n",
		  );
	    print("-------------------------------------\n");
	}
	$req_mflops += $Entry[1];
	push(@ProcList, @Entry);
    }
    close(ProcFile);
    $nProcs = @ProcList / proc_ncol();
    print (tStamp(), " Total $nProcs processes $req_mflops MFlops \n");
}

#===========================================================================
#
#    Subroutine:  procParse
#
#    Arguments:   A process list command line
#
#    Returns:     A list comprising a process entry.
#
#    Purpose:     Parse a process definition line. The process definition 
#                 parsed and a process entry is produced. The general form
#                 for a process definition is as follows:
#                     [-name:<name>] [-when:<when-condition>] 
#                                    [-cd:<directory>] [-log[:<log-file>]] 
#                                    [-nlives:<n-restart>]
#                                    <command>
#                 where:
#                     <name>           the process name
#                     <when-condition> an sh command which if it evaluates 
#                                      to 0 (no error) will enable the 
#                                      process to be run.
#                     <log-file>       a log file into which the standard
#                                      output will be copied. If -log is
#                                      specified without a file name, the 
#                                      $HOME/logs/<name>-%t.log is used.
#                     <n-restart>      Number of restarts
#                     <command>        the shell command to be executed.
#
#===========================================================================
sub procParse {
    my $Line = $_[0];
    return () if (! length($Line));
    $Line =~ s/\s(\s*)/ /g;
    $Line =~ s/^ //;
    while($Line =~ s/(.*?) (.*?)/$1#$2/g) { }
    $Line =~ s/"(.*?)"/$1/g;
    my @List = split(/ /,$Line);
    for (my $i=0 ; $i < @List ; $i++) {
	$List[$i] =~ s/#/ /g;
    }
    my $Wait   = 0;
    my $Name   = "";
    my $Log    = "";
    my $When   = "";
    my $Resets = $MaxRestart;
    my $Cwd    = "$ENV{DMTSTORE}/pars";
    while ($List[0] =~ "^-") {
	my $Key = shift(@List);
	print "Key: $Key \n" if($Debug);
	if ($Key eq "-wait") {
	    $Wait |= 1;
	} elsif ($Key eq "-hup") {
	    $Wait |= 2;
	} elsif ($Key =~ "^-cd:") {
	    $Cwd = substr($Key, 4);
	} elsif ($Key =~ "^-name:") {
	    $Name = substr($Key, 6);
	} elsif ($Key =~ "^-nlives:") {
	    $Resets = substr($Key, 8);
	} elsif ($Key =~ "^-when:") {
	    $When = substr($Key, 6);
	} elsif ($Key eq "-log") {
	    $Log = "<default>";
	} elsif ($Key =~ "^-log:") {
	    $Log = substr($Key, 5);
	} elsif ($Key =~ "^-no-output:") {
	    my @OutList = split(':', substr($Key, 11));
	    $Wait |= 0x04 if (scalar(grep(/trend/, @OutList)) > 0);
	    $Wait |= 0x08 if (scalar(grep(/html/,  @OutList)) > 0);
	    $Wait |= 0x10 if (scalar(grep(/data/,  @OutList)) > 0);
	} elsif ($Key eq "-no-output") {
	    $Wait |= 0x1c;
	} else {
	    warn "Unidentified option: $Key\n";
	    return ();
	}
    }
    my $Cmd = join(' ', @List);
    if (! $Name) {
	$Name = $List[0];
	$Name =~ s|.*/||g;
	print "Process assigned name $Name \n" if ($Debug);
    }
    $Log = "\$DMTSTORE/logs/${Name}-\%t.log" if ($Log eq "<default>");
    return ($Name, $Cmd, 0, $Resets, $Log, $Wait, $When, 0, $Cwd);
}
#
1;
